@@ -461,7 +461,8 @@ sub _checkConfig
     if (defined($conf->{rewrite})) {
         foreach my $rule (@{$conf->{rewrite}}) {
             if (defined($rule->{filter})) {
-                $rule->{filterobj} = Net::LDAP::Filter->new($rule->{filter});
+                $rule->filter =~ s/&amp;/&/g;
+                $rule->{filterobj} = Net::LDAP::Filter->new(encode('utf8', $rule->{filter}));
             }
             if (defined($rule->{attrs})) {
                 $rule->{attrs} =~ s/,/|/;
@@ -482,9 +483,23 @@ sub _rewritePattern
     my $self = shift;
     my ($str, $pattern, $value) = @_;
 
-    my ($qt) = ($str =~ /(['"])$pattern/);
-    if ($str =~ /$qt$pattern$qt/) {
-        $value =~ s/$qt/\\$qt/g;
+    my @rwmaps = ($str =~ /%{([^(]*\((?:(?!%{).)*\))}/gs);
+    foreach my $rwmap (@rwmaps) {
+        my $tmpstr = $rwmap;
+        my $tmpval = $value;
+        my $qt = '';
+        if ($rwmap =~ /'[^']*$pattern[^']*'/) {
+            $qt = '\'';
+        } elsif ($rwmap =~ /"[^"]*$pattern[^"]*"/) {
+            $qt = '"';
+        }
+
+        if ($qt) {
+            $tmpval =~ s/$qt/\\$qt/g;
+            $tmpstr =~ s/$pattern/$tmpval/g;
+        }
+        $rwmap =~ s/([.*+?\[\]()|\^\$\\])/\\$1/g;
+        $str =~ s/%{$rwmap}/%{$tmpstr}/;
     }
 
     $str =~ s/$pattern/$value/g;
