@@ -289,8 +289,9 @@ sub _objSearch
             my $poconf = $self->{object}{$oconf->{container}[0]->{oname}[0]}->{conf};
 
             if ($oconf->{table}[0] ne $poconf->{table}[0]) {
+                my $idquote = !defined($poconf->{id}[0]->{type}) || $poconf->{id}[0]->{type}[0] !~ /^(int|smallint|float|number)$/i ? "'" : '';
                 $from = "$from, $poconf->{table}[0]";
-                $where = "$where and $poconf->{table}[0].$poconf->{id}[0]->{column}[0] ".(defined($pkey) ? "= \'$pkey\'" : "is NULL");
+                $where = "$where and $poconf->{table}[0].$poconf->{id}[0]->{column}[0] ".(defined($pkey) ? "= $idquote$pkey$idquote" : "is NULL");
             }
 
             if (defined($oconf->{container}[0]->{fromtbls})) {
@@ -480,6 +481,7 @@ sub _objModify
             last DO;
         }
 
+        my $idquote = !defined($oconf->{id}[0]->{type}) || $oconf->{id}[0]->{type}[0] !~ /^(int|smallint|float|number)$/i ? "'" : '';
         while ( @list > 0 ) {
             my $action = shift @list;
             my $attr    = lc(shift @list);
@@ -521,6 +523,7 @@ sub _objModify
                 }
             }
 
+            my $quote = !defined($oconf->{attr}{$attr}->{type}) || $oconf->{attr}{$attr}->{type}[0] !~ /^(int|smallint|float|number)$/i ? "'" : '';
             if($action eq "ADD") {
                 # check whether the value already exists
                 my $vals_str = "(".join('|', @values).")";
@@ -543,7 +546,7 @@ sub _objModify
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                         last DO;
                     }
-                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'$value\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
+                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = $quote$value$quote where $oconf->{id}[0]->{column}[0] = $idquote$key$idquote";
 
                     # multibyte value
                     $sql = encode($conf->{mbcode}[0], $sql);
@@ -588,7 +591,7 @@ sub _objModify
                             $value = join($oconf->{attr}{$attr}->{delim}[0], @new_vals);
                         }
                     }
-                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'$value\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
+                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = $quote$value$quote where $oconf->{id}[0]->{column}[0] = $idquote$key$idquote";
 
                     # multibyte value
                     $sql = encode($conf->{mbcode}[0], $sql);
@@ -627,7 +630,7 @@ sub _objModify
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                         last DO;
                     }
-                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'$value\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
+                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = $quote$value$quote where $oconf->{id}[0]->{column}[0] = $idquote$key$idquote";
 
                     # multibyte value
                     $sql = encode($conf->{mbcode}[0], $sql);
@@ -758,23 +761,24 @@ sub _objAdd
             $value =~ s/'/''/g;
             $value =~ s/\\/\\\\/g;
 
+            my $quote = !defined($oconf->{attr}{$attr}->{type}) || $oconf->{attr}{$attr}->{type}[0] !~ /^(int|smallint|float|number)$/i ? "'" : '';
             if (defined($oconf->{attr}{$attr}->{column})) {
                 if (defined($oconf->{attr}{$attr}->{delim})) {
                     my $match = 0;
                     for (my $i = 0; $i < @cols; $i++) {
                         if ($cols[$i] =~ /^$oconf->{attr}{$attr}->{column}[0]$/) {
-                            $values[$i] =~ s/^'(.+)'$/\'$1$oconf->{attr}{$attr}->{delim}[0]$value\'/;
+                            $values[$i] =~ s/^$quote(.+)$quote$/$quote$1$oconf->{attr}{$attr}->{delim}[0]$value$quote/;
                             $match = 1;
                             last;
                         }
                     }
                     if (!$match) {
                         push(@cols, $oconf->{attr}{$attr}->{column}[0]);
-                        push(@values, "\'$value\'");
+                        push(@values, $quote.$value.$quote);
                     }
                 } elsif (!grep(/^$oconf->{attr}{$attr}->{column}[0]$/, @cols)) {
                     push(@cols, $oconf->{attr}{$attr}->{column}[0]);
-                    push(@values, "\'$value\'");
+                    push(@values, $quote.$value.$quote);
                 }
             }
             if (defined($oconf->{attr}{$attr}->{addproc})) {
@@ -788,10 +792,11 @@ sub _objAdd
             my $value = $self->_getStaticValue($strginfo, $dn, $entryStr);
             push(@si_values, $value);
 
+            my $quote = !defined($strginfo->{type}) || $strginfo->{type}[0] !~ /^(int|smallint|float|number)$/i ? "'" : '';
             if (defined($strginfo->{column})) {
                 if (!grep(/^$strginfo->{column}[0]$/, @cols)) {
                     push(@cols, $strginfo->{column}[0]);
-                    push(@values, "\'$value\'");
+                    push(@values, $quote.$value.$quote);
                 }
             }
         }
@@ -984,8 +989,9 @@ sub _objDelete
                 $sth->finish;
             }
 
+            my $idquote = !defined($oconf->{id}[0]->{type}) || $oconf->{id}[0]->{type}[0] !~ /^(int|smallint|float|number)$/i ? "'" : '';
             # delete the appropriate record from the object's table
-            $sql = "delete from $oconf->{table}[0] where $oconf->{id}[0]->{column}[0] = \'$key\'";
+            $sql = "delete from $oconf->{table}[0] where $oconf->{id}[0]->{column}[0] = $idquote$key$idquote";
             $sth = $self->{db}->prepare($sql);
             if (!$sth->execute) {
                 $self->log(level => 'err', message => "Deleting entry by \"$sql\" failed: ".$sth->errstr);
@@ -1130,7 +1136,8 @@ sub _getParentRdn
         $from = "$from,$oconf->{container}[0]->{fromtbls}[0]";
     }
 
-    $where = "$oconf->{table}[0].$oconf->{id}[0]->{column}[0] = \'$key\'";
+    my $idquote = !defined($oconf->{id}[0]->{type}) || $oconf->{id}[0]->{type}[0] !~ /^(int|smallint|float|number)$/i ? "'" : '';
+    $where = "$oconf->{table}[0].$oconf->{id}[0]->{column}[0] = $idquote$key$idquote";
     if (defined($oconf->{container}[0]->{joinwhere})) {
         if ($oconf->{container}[0]->{joinwhere}[0] =~ /\%c/) {
             my ($pidcol) = ($oconf->{container}[0]->{joinwhere}[0] =~ /([^ ]+) *= *'?\%c'?/);
@@ -1220,7 +1227,8 @@ sub _getAttrValues
         $where =~ s/\%o/$key/g;
         $where =~ s/\%p/$pkey/g;
     } else {
-        $where = "$oconf->{table}[0].$oconf->{id}[0]->{column}[0] = \'$key\'";
+        my $idquote = !defined($oconf->{id}[0]->{type}) || $oconf->{id}[0]->{type}[0] !~ /^(int|smallint|float|number)$/i ? "'" : '';
+        $where = "$oconf->{table}[0].$oconf->{id}[0]->{column}[0] = $idquote$key$idquote";
         if (defined($oconf->{attr}{$attr}->{joinwhere})) {
             $where = "$where and $oconf->{attr}{$attr}->{joinwhere}[0]";
             $where =~ s/\%p/$pkey/g;
@@ -1502,6 +1510,7 @@ sub _filter2sql
     $value =~ s/'/''/g;
     $value =~ s/\\/\\\\/g;
 
+    my $quote = !defined($oconf->{attr}{$attr}->{type}) || $oconf->{attr}{$attr}->{type}[0] !~ /^(int|smallint|float|number)$/i ? "'" : '';
     ${$from} = $oconf->{table}[0];
     if ($attr =~ /^objectclass$/i) {
         # check object class in filter
@@ -1516,7 +1525,7 @@ sub _filter2sql
         # attribute doesn't exit
         ${$where} = '1 = 0';
     } elsif (defined($oconf->{attr}{$attr}->{column})) {
-        ${$where} = "lower($oconf->{table}[0].$oconf->{attr}{$attr}->{column}[0]) $type lower(\'$value\')";
+        ${$where} = "lower($oconf->{table}[0].$oconf->{attr}{$attr}->{column}[0]) $type ".($quote ? "lower($quote$value$quote)" : "$quote$value$quote");
     } elsif (defined($oconf->{attr}{$attr}->{oname})) {
         my $aconf = $self->{object}{$oconf->{attr}{$attr}->{oname}[0]}->{conf};
 
@@ -1528,7 +1537,8 @@ sub _filter2sql
         }
 
         ($value) = ($value =~ /^$aconf->{rdn}[0]=([^,]*)/i);
-        my $subquery = "select $oconf->{table}[0].$oconf->{id}[0]->{column}[0] from ${$from} where lower($aconf->{table}[0].$aconf->{attr}{$aconf->{rdn}[0]}->{column}[0]) $type lower(\'$value\')";
+        my $subquery = "select $oconf->{table}[0].$oconf->{id}[0]->{column}[0] from ${$from} where lower($aconf->{table}[0].$aconf->{attr}{$aconf->{rdn}[0]}->{column}[0]) $type ".($quote ? "lower($quote$value$quote)" : "$quote$value$quote")
+;
         if (defined($oconf->{attr}{$attr}->{joinwhere})) {
             $subquery = "$subquery and $oconf->{attr}{$attr}->{joinwhere}[0]";
         }
@@ -1553,7 +1563,7 @@ sub _filter2sql
             ${$from} = "${$from},$oconf->{attr}{$attr}->{fromtbls}[0]";
         }
 
-        my $subquery =  "select $oconf->{table}[0].$oconf->{id}[0]->{column}[0] from ${$from} where lower($oconf->{attr}{$attr}->{selexpr}[0]) $type lower(\'$value\')";
+        my $subquery =  "select $oconf->{table}[0].$oconf->{id}[0]->{column}[0] from ${$from} where lower($oconf->{attr}{$attr}->{selexpr}[0]) $type ".($quote ? "lower($quote$value$quote)" : "$quote$value$quote");
         if (defined($oconf->{attr}{$attr}->{joinwhere})) {
             $subquery = "$subquery and $oconf->{attr}{$attr}->{joinwhere}[0]";
         }
