@@ -203,8 +203,31 @@ class AutoLogin
     function start()
     {
         if (isset($_SERVER['REMOTE_USER_LOGGEDIN'])) {
-            header('Location: '.$this->url['back']);
-            return true;
+            $loggedin = true;
+            if (isset($this->url['session'])) {
+                $loggedin = false;
+                $req =& new HTTP_Request($this->url['session']);
+                $req->addHeader('User-Agent', $_SERVER['HTTP_USER_AGENT']);
+                $req->setMethod(HTTP_REQUEST_METHOD_GET);
+                foreach ($_COOKIE as $key => $value) {
+                    $req->addCookie($key, $value);
+                }
+                $res = $req->sendRequest();
+                if (!PEAR::isError($res)) {
+                    $response = $req->getResponseCode();
+                    if ($response < 300) {
+                        $body = $req->getResponseBody();
+                        if (!isset($this->content['sessionfail']) || !preg_match('/'.$this->content['sessionfail'].'/', $body)) {
+                            $loggedin = true;
+                        }
+                    }
+                }
+            }
+            if ($loggedin) {
+                $back = isset($this->url['back']) ? $this->url['back'] : $this->url['login'];
+                header('Location: '.$back);
+                return true;
+            }
         }
 
         if (PEAR::isError($rc = $this->login())) {
@@ -770,8 +793,7 @@ class AutoLogin
         } else if (preg_match('/^\//', $url)) {
             return preg_replace('/^(https?:\/\/[^\/]+).*$/', '$1', $oldurl).$url;
         } else {
-            preg_replace('/[^\/]+$/', '', $oldurl);
-            return $oldurl.$url;
+            return preg_replace('/[^\/]+$/', '', $oldurl).$url;
         }
     }
 
